@@ -143,7 +143,7 @@ generate.mnemonics <- function(stats, hierarchical = FALSE, clust.func = hclust,
 ## are replaced by the string at [i,2])
 ## If both a translation table and regex replacements are specified, the
 ## translation table is applied first
-rename.tracks <- function(stats, patterns = "_", replacements = ".",
+rename.tracks <- function(stats, patterns = NULL, replacements = NULL,
                           translation = NULL, ...) {
   if (!is.data.frame(stats)) {
     stop("rename.tracks expected stats as data.frame")
@@ -250,9 +250,10 @@ normalize.binary.track.stats <- function(stats, cov = FALSE) {
 hierarchical.dist <- function(mat) {
   ## Returns a distance matrix, accomodating a hierarchical model
   
-  ## Hierarchy is denoted in the rownames, with "a_b" specifying a's child, b
+  ## Hierarchy is denoted in the rownames, with "a_b" or "a.b"
+  ## specifying a's child, b
   ## All nodes of a given level must have the same number of children
-  levels.tokens <- strsplit(rownames(mat), "_", fixed = TRUE)
+  levels.tokens <- strsplit(rownames(mat), "[_.]")
   depth <- length(levels.tokens[[1]])
   for (level.row in levels.tokens) {
     if (length(level.row) != depth) {
@@ -405,7 +406,9 @@ panel.track.stats <-
 }
 
 levelplot.track.stats <-
-  function(track.stats,
+  function(stats.obj = NULL,
+           track.stats = stats.obj[["stats"]],
+           hierarchical = stats.obj[["hierarchical"]],
            axis.cex = 1.0,
            scale.cex = 1.0,
            xlab = list("Segment label", cex = axis.cex),
@@ -413,8 +416,7 @@ levelplot.track.stats <-
            hclust.label = TRUE,
            hclust.track = TRUE,
            clust.func = hclust,
-           hierarchical = FALSE,
-           aspect = "iso",
+           aspect = "fill",
            sd.shape = "line",
            box.fill = NULL,
            symmetric = FALSE,
@@ -431,6 +433,9 @@ levelplot.track.stats <-
   if (!is.array(track.stats)) {
     track.stats <- as.stats.array(track.stats)
   }
+  if (is.null(hierarchical)) {
+    hierarchical <- FALSE
+  }
 
   if (dim(track.stats[, , "mean"])[2] == 2) {
     norm.func <- normalize.binary.track.stats
@@ -507,42 +512,70 @@ levelplot.track.stats <-
             ...)
 }
 
-plot.track.stats <- function(filename, mnemonics = NULL,
-                             translation_file = NULL,
-                             gmtk = FALSE, symmetric = FALSE, ...) {
-  translation <-
-    if (length(translation_file) > 0 && nchar(translation_file) > 0) {
-      read.mnemonics(translation_file)
-    } else {
-      NULL
-    }
-  
+load.track.stats <- function(filename, mnemonics = NULL,
+                             translations = NULL,
+                             as_regex = FALSE,  # Translate with regexs
+                             gmtk = FALSE,
+                             ...) {
   if (gmtk) {
-    stats <- read.gmtk.track.stats(filename, mnemonics = mnemonics,
-                                   translation = translation, ...)
+    read.func <- read.gmtk.track.stats
     hierarchical <- is.hierarchical.gmtk(filename)
   } else {
-    stats <- read.track.stats(filename, mnemonics = mnemonics,
-                              translation = translation, ...)
+    read.func <- read.track.stats
     hierarchical <- FALSE
   }
+
+  args <- list(filename, mnemonics = mnemonics, ...)
+  if (as_regex) {
+    args$patterns <- translations$old
+    args$replacements <- translations$new
+  } else {
+    args$translations <- translations
+  }
   
-  levelplot.track.stats(stats, hierarchical = hierarchical,
-                        symmetric = symmetric, ...)
+  stats <- do.call(read.func, args)
+
+  list(stats = stats, hierarchical = hierarchical)
+}
+
+plot.track.stats <- function(filename, symmetric = FALSE, ...) {
+  res <- load.track.stats(filename, ...)
+  levelplot.track.stats(res, symmetric = symmetric, ...)
+}
+
+save.track.stats <- function(dirpath, namebase, filename,
+                             mnemonic_file = NULL,
+                             translation_file = NULL,
+                             symmetric = FALSE, 
+                             clobber = FALSE,
+                             square.size = 15,  # px
+                             ...) {
+  mnemonics <- read.mnemonics(mnemonic_file)
+  translations <- read.mnemonics(translation_file)
+  res <- load.track.stats(filename,
+                          mnemonics = mnemonics,
+                          translations = translations,
+                          ...)
+
+  ntracks <- nlevels(res$stats$trackname)
+  nlabels <- nlevels(res$stats$label)
+  height <- 200 + square.size * ntracks
+  width <- 450 + square.size * nlabels
+
+  save.images(dirpath, namebase,
+              levelplot.track.stats(res, symmetric = symmetric, ...),
+              height = height,
+              width = width,
+              clobber = clobber)
 }
 
 ## infilename: stats data.frame tab file
 ## outfilename: name of mnemonic file to create
 make.mnemonic.file <- function(infilename, outfilename, gmtk = FALSE, ...)
 {
-  if (gmtk) {
-    stats <- read.gmtk.track.stats(infilename, ...)
-    hierarchical <- is.hierarchical.gmtk(infilename)
-  } else {
-    stats <- read.track.stats(infilename, ...)
-    hierarchical <- FALSE
-  }
-  stats <- as.stats.array(stats)
+  res <- load.track.stats(infilename, gmtk = gmtk, ...)
+  hierarchical <- res$hierarchical
+  stats <- as.stats.array(res$stats)
   stats.norm <- normalize.track.stats(stats)
   mnemonics <- generate.mnemonics(stats.norm, hierarchical = hierarchical, ...)
   write.table(mnemonics, file = outfilename, quote = FALSE, 
