@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 from __future__ import division, with_statement
 
-__version__ = "$Revision: 192 $"
+__version__ = "$Revision: 238 $"
 
 """
 html.py
@@ -14,10 +14,13 @@ import re
 import sys
 import time
 
+from functools import partial
+from pkg_resources import resource_string
 from shutil import copy
+from string import Template
 
-from . import Segmentation
-from .common import check_clobber, die, make_divfilename, make_id, make_filename, make_tabfilename, NICE_EXTS, template_substitute
+from . import log, Segmentation
+from .common import check_clobber, die, get_ordered_labels, make_divfilename, make_id, make_filename, make_tabfilename, map_mnemonics, NICE_EXTS, PKG_RESOURCE
 
 MNEMONIC_TEMPLATE_FILENAME = "mnemonic_div.tmpl"
 HEADER_TEMPLATE_FILENAME = "html_header.tmpl"
@@ -32,6 +35,18 @@ GENOMEBROWSER_LINK_TMPL = """
 <li>Link to view this segmentation in the UCSC genome browser:<br />
 <script type="text/javascript">print_genomebrowser_link("%s");</script>
 </li>"""
+
+DESCRIPTION_MODULE = ("description", "Segmentation information")
+MNEMONIC_MODULE = ("mnemonics", "Mnemonics")
+
+template_string = partial(resource_string, PKG_RESOURCE)
+
+def template_substitute(filename):
+    """
+    Simplify import resource strings in the package
+    """
+    return Template(template_string(filename)).safe_substitute
+
 def tuple2link(entry):
     """entry should be a (url, text) tuple"""
     return '<a href="%s">%s</a>' % entry
@@ -40,7 +55,7 @@ def list2html(list, code=False, link=False):
     """
     If link is True: each element of the list should be a (divID, label) tuple
     """
-    result = "<ul>"
+    result = ["<ul>"]
     entrystr = "%s"
     if link:
         entrystr = '<a href="#%s">%s</a>'
@@ -49,35 +64,64 @@ def list2html(list, code=False, link=False):
 
     entrystr = "<li>%s</li>" % entrystr
     for entry in list:
-        result += entrystr % entry
-    return result + "</ul>"
+        result.append(entrystr % entry)
 
-def tab2html(tabfile, header=True):
+    result.append("</ul>")
+    return "\n".join(result)
+
+def tab2html(tabfile, header=True, mnemonicfile=None):
     """
     Given a tab file table with a header row, generates an html string which
     for pretty-display of the table data.
     """
-    result = '\n<table border="1" cellpadding="4" cellspacing="1">'
+    if not os.path.isfile(tabfile):
+        return "<File not found>"
+
+    result = []
+    result.append('\n<table border="1" cellpadding="4" cellspacing="1">')
     # if the tabfile exists, write in htmlhandle an html table
     with open(tabfile) as ifp:
-        if header:
-            # first read the first line, and write the header row
+        # Read past comments
+        line = ifp.readline()
+        while line.startswith("#"):
             line = ifp.readline()
-            fields = line.split("\t")
-            result += "<tr>"
-            for f in fields:
-                result += '<td style="background-color: \
-rgb(204, 204, 204)">%s</td>' % f
-            result += "</tr>"
 
-        for line in ifp:
-            result += "<tr>"
+        if header:
+            # Write colname row from header row of file
             fields = line.split("\t")
+            result.append("<tr>")
+
             for f in fields:
-                result += "<td>%s</td>" % f
-            result += "</tr>"
-    result += "</table>\n"
-    return result
+                result.append('<td style="background-color:'
+                             'rgb(204, 204, 204)">%s</td>' % f)
+
+            result.append("</tr>")
+            line = ifp.readline()
+
+        lines = [line] + ifp.readlines()
+        rows = [line.split("\t") for line in lines]
+        row_names = [row[0] for row in rows]
+        # Make basic labels for row names
+        row_order = range(0, len(rows))
+        row_labels = dict(zip(row_order, row_names))
+        if mnemonicfile:
+            # Substitute these labels with mnemonics
+            mnemonics = map_mnemonics(row_labels, mnemonicfile)
+            row_order, row_labels = get_ordered_labels(row_labels,
+                                                       mnemonics)
+
+        for row_key in row_order:
+            entry = ["<tr>"]
+            fields = rows[row_key]
+            fields[0] = row_labels[row_key]  # Replace row_name
+            for f in fields:
+                entry.append("<td>%s</td>" % f)
+
+            entry.append("</tr>")
+            result.append("".join(entry))
+
+    result.append("</table>\n")
+    return "\n".join(result)
 
 def find_output_files(dirpath, namebase, d={}, tag=""):
     exts = NICE_EXTS
@@ -91,8 +135,8 @@ def find_output_files(dirpath, namebase, d={}, tag=""):
 
     return d
 
-def form_template_dict(dirpath, namebase, module=None,
-                       extra_namebases={}, **kwargs):
+def form_template_dict(dirpath, namebase, module=None, extra_namebases={},
+                       mnemonicfile=None, tables={}, **kwargs):
     """
     Given information about the current validation, generates a dictionary
     suitable for HTML template substitution.
@@ -110,11 +154,14 @@ def form_template_dict(dirpath, namebase, module=None,
     an id variable will be generated based upon the module and dirpath
     (a pseudo-unique identifier for the div file).
 
-    Any keyword arguments of the form <prefix>tablenamebase
-    will be assumed to refer to a table tab files and will be converted to
-    html form and linked in under a variable named <prefix>table.
-    The corresponding tab file is linked under a variable named
-    <prefix>tablefilename.
+    tables: a dict from tag -> to table file namebase string.
+    The rownames of the tables will try to be substituted with the mnemonics
+    in mnemonicfile, if provided. Variable of the form <tag>table and
+    <tag>tablefilename will be created with the table HTML and table file
+    name, respectively. The dict value can also be a tuple of two elements,
+    where the first is the namebase string and the second is a string
+    mode. Currently, only None and "exact" are supported,
+    with the latter causing the table to not be mnemonic-substituted.
 
     Any other keyword args supplied are linked into the dictionary.
     """
@@ -129,21 +176,29 @@ def form_template_dict(dirpath, namebase, module=None,
         assert arg not in d
         d[arg] = make_id(module, dirpath)
 
+    # Add any tables
+    for tag, table in tables.iteritems():
+        if isinstance(table, tuple):
+            table, tablemode = table
+        else:
+            tablemode = None
+
+        tablefilename = make_tabfilename(dirpath, table)
+        filearg = "%stablefilename" % tag
+        tablearg = "%stable" % tag
+        if tablemode is None:
+            val = tab2html(tablefilename, mnemonicfile=mnemonicfile)
+        elif tablemode == "exact":
+            val = tab2html(tablefilename)
+        else:
+            raise ValueError("Invalid table mode: %s" % tablemode)
+
+        assert filearg not in d and tablearg not in d  # Don't overwrite
+        d[filearg] = tablefilename
+        d[tablearg] = val
+
     for arg, val in kwargs.iteritems():
-        index = arg.rfind("tablenamebase")
-        if index >= 0: # arg ends with "tablenamebase"
-            tablefilename = make_tabfilename(dirpath, val)
-            if os.path.isfile(tablefilename):
-                # Save html table under variable
-                prefix = arg[0:index]
-                arg = "%stable" % prefix
-                val = tab2html(tablefilename)
-                # Save table tab file under separate variable
-                filearg = "%stablefilename" % prefix
-                assert filearg not in d
-                d[filearg] = tablefilename
-            # else leave arg and val the way they were
-        assert arg not in d  # Make sure wer're not overwriting anything
+        assert arg not in d  # Don't overwrite
         d[arg] = val
 
     return d
@@ -156,21 +211,22 @@ def write_html_div(dirpath, namebase, html, clobber=False):
     check_clobber(filename, clobber)
 
     with open(filename, "w") as ofp:
-        print >>ofp, html
+        ofp.write("%s\n" % html)
 
 def save_html_div(template_filename, dirpath, namebase,
                   clobber=False, **kwargs):
+    """Save an HTML div file for a module run by subsituting a template file"""
     fields = form_template_dict(dirpath, namebase, **kwargs)
 
     try:
         html = template_substitute(template_filename)(fields)
     except KeyError, e:
-        print >>sys.stderr, "Skipping HTML output. Missing data: %s" % e
+        log("Skipping HTML output. Missing data: %s" % e)
         return
 
     write_html_div(dirpath, namebase, html, clobber=clobber)
 
-def form_mnemonic_div(mnemonicfile, results_dir, clobber=False):
+def form_mnemonic_div(mnemonicfile, results_dir, clobber=False, verbose=True):
     """Copy mnemonic file to results_dir and create the HTML div with a link"""
     filebase = os.path.basename(mnemonicfile)
     link_file = os.path.join(results_dir, filebase)
@@ -179,11 +235,11 @@ def form_mnemonic_div(mnemonicfile, results_dir, clobber=False):
     try:
         copy(mnemonicfile, link_file)
     except (IOError, os.error):
-        print >>sys.stderr, ("Error: could not copy %s to %s. Linking"
-                             " the the former." % (mnemonicfile, link_file))
+        log("Error: could not copy %s to %s. Linking"
+            " the the former." % (mnemonicfile, link_file))
         link_file = mnemonicfile  # Link to the existing file
     else:
-        print >>sys.stderr, "Copied %s to %s" % (mnemonicfile, link_file)
+        log("Copied %s to %s" % (mnemonicfile, link_file), verbose)
 
     fields = {}
     fields["tabfilename"] = link_file
@@ -259,7 +315,7 @@ def form_html_header(bedfilename, modules, layeredbed=None, bigbed=None):
 def form_html_footer():
     return template_substitute(FOOTER_TEMPLATE_FILENAME)()
 
-def find_divs(rootdir=os.getcwd()):
+def find_divs(rootdir=os.getcwd(), verbose=True):
     """Look one level deep in directory, adding any .div files found.
     """
     divs = []
@@ -271,46 +327,94 @@ def find_divs(rootdir=os.getcwd()):
                 if ext == ".div":
                     filepath = os.path.join(folderpath, filename)
                     divs.append(filepath)
-                    print >>sys.stderr, "Found div: %s" % filename
+                    log("Found div: %s" % filename, verbose)
     return divs
 
+def make_html_report(bedfilename, results_dir, outfile, mnemonicfile=None,
+                     clobber=False, verbose=True,
+                     layeredbed=None, bigbed=None):
+    check_clobber(outfile, clobber)
+
+    divs = find_divs(results_dir, verbose=verbose)
+    if len(divs) == 0:
+        die("Unable to find any module .div files."
+            " Make sure to run this from the parent directory of the"
+            " module output directories or specify the --results-dir option")
+
+    body = []
+    modules = [DESCRIPTION_MODULE]
+    if mnemonicfile is not None:
+        modules.append(MNEMONIC_MODULE)
+        body.append(form_mnemonic_div(mnemonicfile, results_dir, clobber=clobber, verbose=verbose))
+
+    regex = re.compile('"module" id="(.*?)".*?<h.>.*?</a>\s*(.*?)\s*</h.>',
+                       re.DOTALL)
+    for div in divs:
+        with open(div) as ifp:
+            divstring = "".join(ifp.readlines())
+            matching = regex.search(divstring)
+            assert matching
+            module = (matching.group(1), matching.group(2))
+            modules.append(module)
+            body.append(divstring)
+
+    header = form_html_header(bedfilename, modules,
+                              layeredbed=layeredbed,
+                              bigbed=bigbed)
+    footer = form_html_footer()
+
+    components = [header] + body + [footer]
+    separator = "<br /><hr>"
+    with open(outfile, "w") as ofp:
+        ofp.write("%s\n" % separator.join(components))
+
 def parse_args(args):
-    from optparse import OptionParser
-    usage = "%prog [OPTIONS] BEDFILE"
+    from optparse import OptionGroup, OptionParser
+    usage = "%prog [OPTIONS] SEGMENTATION"
     version = "%%prog %s" % __version__
     parser = OptionParser(usage=usage, version=version)
 
+    group = OptionGroup(parser, "Flags")
+    group.add_option("--clobber", action="store_true",
+                     dest="clobber", default=False,
+                     help="Overwrite existing output files if the specified"
+                     " directory already exists.")
+    group.add_option("-q", "--quiet", action="store_false",
+                     dest="verbose", default=True,
+                     help="Do not print diagnostic messages.")
+    parser.add_option_group(group)
 
-    parser.add_option("--clobber", action="store_true",
-                      dest="clobber", default=False,
-                      help="Overwrite existing output files if the specified"
-                      " directory already exists.")
-    parser.add_option("--mnemonic-file", dest="mnemonicfile",
-                      default=None, metavar="FILE",
-                      help="If specified, this mnemonic mapping will be"
-                      " included in the report (this should be the same"
-                      " mnemonic file used by the individual modules)")
-    parser.add_option("-L", "--layered-bed", dest="layeredbed",
-                      default=None, metavar="FILE",
-                      help="If specified, this layered BED file will be linked"
-                      " into the the HTML document (assumed to be the same"
-                      " data as in BEDFILE)")
-    parser.add_option("-B", "--big-bed", dest="bigbed",
-                      default=None, metavar="FILE",
-                      help="If specified, this bigBed file will be linked into"
-                      " the the HTML document and a UCSC genome brower link"
-                      " will be generated for it (assumed to be the same data"
-                      " as in BEDFILE)")
-    parser.add_option("--results-dir", dest="resultsdir",
-                      default=".", metavar="DIR",
-                      help="This should be the directory containing all the"
-                      " module output directories (`ls` should return things"
-                      " like \"length_distribution/\", etc)"
-                      " [default: %default]")
-    parser.add_option("-o", "--outfile", metavar="FILE",
-                      dest="outfile", default="index.html",
-                      help="HTML report file (must be in current directory"
-                      " or the links will break [default: %default]")
+    group = OptionGroup(parser, "Linking")
+    group.add_option("-m", "--mnemonic-file", dest="mnemonicfile",
+                     default=None, metavar="FILE",
+                     help="If specified, this mnemonic mapping will be"
+                     " included in the report (this should be the same"
+                     " mnemonic file used by the individual modules).")
+    group.add_option("-L", "--layered-bed", dest="layeredbed",
+                     default=None, metavar="FILE",
+                     help="If specified, this layered BED file will be linked"
+                     " into the the HTML document (assumed to be the same"
+                     " data as in SEGMENTATION)")
+    group.add_option("-B", "--big-bed", dest="bigbed",
+                     default=None, metavar="FILE",
+                     help="If specified, this bigBed file will be linked into"
+                     " the the HTML document and a UCSC genome brower link"
+                     " will be generated for it (assumed to be the same data"
+                     " as in SEGMENTATION)")
+    parser.add_option_group(group)
+
+    group = OptionGroup(parser, "Output")
+    group.add_option("--results-dir", dest="resultsdir",
+                     default=".", metavar="DIR",
+                     help="This should be the directory containing all the"
+                     " module output directories (`ls` should return things"
+                     " like \"length_distribution/\", etc)"
+                     " [default: %default]")
+    group.add_option("-o", "--outfile", metavar="FILE",
+                     dest="outfile", default="index.html",
+                     help="HTML report file (must be in current directory"
+                     " or the links will break [default: %default]")
+    parser.add_option_group(group)
 
     options, args = parser.parse_args(args)
 
@@ -326,43 +430,14 @@ def parse_args(args):
 def main(args=sys.argv[1:]):
     options, args = parse_args(args)
     bedfilename = args[0]
-    mnemonicfile = options.mnemonicfile
     outfile = options.outfile
     results_dir = options.resultsdir
-    clobber = options.clobber
-    check_clobber(outfile, clobber)
-
-    divs = find_divs(results_dir)
-    if len(divs) == 0:
-        die("Unable to find any module .div files."
-            " Make sure to run this from the parent directory of the"
-            " module output directories or specify the --results-dir option")
-
-    body = []
-    modules = []
-    if mnemonicfile is not None:
-        body.append(form_mnemonic_div(mnemonicfile, results_dir, clobber))
-
-    regex = re.compile('"module" id="(.*?)".*?<h.>.*?</a>\s*(.*?)\s*</h.>',
-                       re.DOTALL)
-    for div in divs:
-        with open(div) as ifp:
-            divstring = "".join(ifp.readlines())
-            matching = regex.search(divstring)
-            assert matching
-            module = (matching.group(1), matching.group(2))
-            modules.append(module)
-            body.append(divstring)
-
-    header = form_html_header(bedfilename, modules,
-                              layeredbed=options.layeredbed,
-                              bigbed=options.bigbed)
-    footer = form_html_footer()
-
-    components = [header] + body + [footer]
-    separator = "<br /><hr>"
-    with open(outfile, "w") as ofp:
-        print >>ofp, separator.join(components)
+    kwargs = {"mnemonicfile": options.mnemonicfile,
+              "clobber": options.clobber,
+              "verbose": options.verbose,
+              "layeredbed": options.layeredbed,
+              "bigbed": options.bigbed}
+    make_html_report(bedfilename, results_dir, outfile, **kwargs)
 
 if __name__ == "__main__":
     sys.exit(main())
